#!/usr/bin/ruby 
# -*- coding: utf-8 -*-

require 'optparse'
require 'iconv'

ORCADBNAME = "orca"
BACKUP_SUFIX = "_backup"
TEMP_SUFIX = "_temp"

ERRFILE = "/tmp/jma-receipt_db_check.log"

UTF8MJ = [
 ["\xE2\x80\x94","\xE2\x80\x95"],
 ["\xE3\x80\x9C","\xEF\xBD\x9E"],
 ["\xE2\x80\x96","\xE2\x88\xA5"],
 ["\xE2\x88\x92","\xEF\xBC\x8D"],
 ["\xC2\xA2","\xEF\xBF\xA0"],
 ["\xC2\xA3","\xEF\xBF\xA1"],
 ["\xCS\xAC","xEF\xBF\xA2"]
]


def createdb_utf8(dbname)
  system("createdb -lC -Ttemplate0 -EUTF-8 #{dbname}")
end

def createdb_eucjp(dbname)
  system("createdb -lC -Ttemplate0 -EEUC-JP #{dbname}")
end

def checkdb(dbname, toeuc, lcheck)
  checkcode = "EUC-JISX0213"
  if toeuc
    checkcode = "ISO-2022-JP"
  end
  err_file = open(ERRFILE,"w")
  err_count = 0
  sql = "SELECT pg_encoding_to_char(encoding), datcollate, datctype FROM pg_database WHERE datname = '#{dbname}';"
  result = `psql template1 -A -t -c \"#{sql}\"`
  encoding,collate,ctype = result.chomp.split('|')
  printf("データベース [%s]\n", dbname)
  printf("エンコーディング [%s]\n",encoding)
  printf("ロケール [%s] [%s]\n",collate, ctype)
  printf("\n")
  if ("C" == collate) and ("C" == ctype)
    if encoding == "UTF8"
      printf("OK: 拡張漢字有効\n")
    elsif encoding == "EUC_JP"
      printf("OK: 拡張漢字無効\n")
    else
      printf("encodingが不明です。処理を中止します\n")
      exit 1
    end
  else
    printf("ロケールが C 以外です。ロケールをCで作成し直すことを推奨します。\n")
  end
  printf("\n")
  exit 0 if lcheck
  table_name = ""
  if toeuc  && encoding == "UTF8"
    printf("データベースがEUC-JPに変換可能かチェックします...\n")
  else
    printf("データベースに不正な文字が入っていないかチェックします...\n")
  end
  if encoding == "UTF8"
    iconv = Iconv.new(checkcode, "UTF-8")
    regexp = UTF8MJ.map {|i,j| j}.join("|")
    IO.popen("pg_dump #{dbname}", "r+") {|pipe|
      while line = pipe.gets
        if /^COPY (\S+)/ =~ line
          table_name = $1
        end
        begin
          if toeuc
            UTF8MJ.each {|i,j|
              line.gsub!(Regexp.new(j),i)
            }
            line.gsub!(/\xE2\x80\x94/, "\x22\x12")
          end
          if line =~ Regexp.new(regexp)
            err_file.printf("#{table_name}\n")
            err_file.printf("ERROR1: %s〓%s%s\n", $~.pre_match, $~, $~.post_match)
            err_count += 1
          end
          iconv.iconv(line)
        rescue Iconv::Failure => ex
          err_file.printf("#{table_name}\n")
          err_file.printf("ERROR2: %s〓%s\n", Iconv.iconv("UTF-8", checkcode, ex.success), ex.failed)
          err_count += 1
        end
      end
    }
    iconv.close
  elsif encoding == "EUC_JP"
    iconv = Iconv.new("ISO-2022-JP", "EUC-JP")
    IO.popen("pg_dump #{dbname}", "r+") {|pipe|
      while line = pipe.gets
        if /^COPY (\S+)/ =~ line
          table_name = $1
        end
        begin
          iconv.iconv(line)
        rescue Iconv::Failure => ex
          err_file.printf("#{table_name}\n")
          err_file.printf("ERROR2: %s〓%s\n", Iconv.iconv("UTF-8", "ISO-2022-JP", ex.success), Iconv.iconv("UTF-8", "EUC-JP-MS", ex.failed))
          err_count += 1
        end
      end
    }
    iconv.close
  end
  printf("\n")
  if err_count > 0
    printf("ERROR: データベースに不正な文字がありました\n")
    printf("ERROR: #{ERRFILE} を参照して修正する必要があります\n")
  else
    printf("OK: 不正な文字はありませんでした\n")
  end
end

def dropdb(dbname)
  system("dropdb #{dbname}")
end

def dbexist?(dbname)
  sql = "SELECT count(*) FROM pg_database WHERE datname = '#{dbname}';"
  result = `psql template1 -A -t -c \"#{sql}\"`
  return true if result.chomp.to_i == 1
end

def other_session?(dbname)
  sql = "SELECT count(*) FROM pg_stat_activity WHERE datname = \'#{dbname}\';"
  result = `psql template1 -A -t -c \"#{sql}\"`
  return true if result.chomp.to_i > 0
end

def renamedb(oldname,newname)
  sql = "ALTER DATABASE #{oldname} RENAME TO #{newname};"
  puts "psql template1 -A -c \"#{sql}\""
  system("psql template1 -A -c \"#{sql}\"")
end

def dump_restore(oldname, newname)
  begin
    restore = IO.popen("psql --set ON_ERROR_STOP=on #{newname}", "w")
    IO.popen("pg_dump -EUTF-8 #{oldname}", "r+") {|pipe|
      while line = pipe.gets 
        # UTF-8(MS) → UTF-8(JISX0213)
        UTF8MJ.each {|i,j|
          line.gsub!(Regexp.new(j),i)
        }
        restore.puts line
      end
    }
    restore.close_write
  rescue
    restore.flush
    restore.close
    puts "ERROR:", $!
    exit
  end
end

def dump_restore_utf8ms(oldname, newname)
  begin
    restore = IO.popen("psql --set ON_ERROR_STOP=on #{newname}", "w")
    IO.popen("pg_dump -EUTF-8 #{oldname}", "r+") {|pipe|
      while line = pipe.gets 
        # UTF-8(JISX0213) → UTF-8(MS)
        UTF8MJ.each {|i,j|
          line.gsub!(Regexp.new(i),j)
        }
        restore.puts line
      end
    }
    restore.close_write
  rescue => ex
    restore.flush
    restore.close_write
    puts "ERROR: #{ex}"
    exit
  end
end

if $0 == __FILE__
  orcadb = ORCADBNAME
  tempdb = nil
  backupdb = nil
  toeuc = nil
  check = nil
  lcheck = nil
  clean = nil

  opt = OptionParser.new
  opt.banner = "Usage: #{File.basename($0)} -[etluc] [-d DB name] [-b Backup name]"
  opt.on('-b','--back Backup name', 'バックアップデータベース名') {|value|
    backupdb = value
  }
  opt.on('-d','--db DB name','対象データベース名(デフォルト orca)') {|value|
    orcadb = value
  }
  opt.on('-u', '--utf8','UTF-8へ変換（デフォルト)') {|value|
    toeuc = false
  }
  opt.on('-e', '--euc','EUC-JPへ変換') {|value|
    toeuc = true
  }
  opt.on('-t', '--test','データベースのチェック') {|value|
    check = true
  }
  opt.on('-l', '--locale','データベースのロケールチェック') {|value|
    check = true
    lcheck = true
  }
  opt.on('-c', '--clean','変換後元データベースの削除') {|value|
    clean = true
  }
  opt.parse!(ARGV)

  tempdb = orcadb + TEMP_SUFIX
  backupdb = orcadb + BACKUP_SUFIX if backupdb.nil?

  if !dbexist?(orcadb)
    puts "ERROR: データベース #{orcadb} がありません"
    exit 1
  end

  if check
    checkdb(orcadb, toeuc, lcheck)
    exit 0
  end

  if !other_session?(orcadb).nil?
    puts "ERROR: データベース #{orcadb} に接続しているアプリケーションを終了してから実行してください。"
    exit 1
  end

  count = 1
  temptempdb = tempdb
  while dbexist?(tempdb)
    tempdb = temptempdb + "_" + count.to_s
    count += 1
  end

  count = 1
  tempbackup = backupdb
  while dbexist?(backupdb)
    backupdb = tempbackup + "_" + count.to_s
    count += 1
  end

  if toeuc
    createdb_eucjp(tempdb)
    dump_restore_utf8ms(orcadb, tempdb)
  else
    createdb_utf8(tempdb)
    dump_restore(orcadb, tempdb)
  end
  renamedb(orcadb, backupdb)
  renamedb(tempdb, orcadb)
  if clean
    dropdb(backupdb)
  end
  puts "OK"
end


